package cd.casic.module.machine.controller;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.framework.commons.util.object.BeanUtils;
import cd.casic.module.machine.dal.dataobject.MachineInfoDO;
import cd.casic.module.machine.enums.ConnectionStatus;
import cd.casic.module.machine.service.MachineInfoService;
import cd.casic.module.machine.controller.vo.MachineInfoVO;
import cn.hutool.core.collection.CollUtil;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

import static cd.casic.framework.commons.pojo.CommonResult.success;

@RestController
@Tag(name = "机器信息管理")
@RequestMapping("/ci/machineInfo")
@Validated
public class MachineInfoController {
    @Resource
    private MachineInfoService machineInfoService;

    @PostMapping("/create")
    @Operation(summary = "新增机器信息")
//    @PreAuthorize("@ss.hasPermission('ci:machineInfo:create')")
    public CommonResult<Long> createMachine(@Valid @RequestBody MachineInfoVO machineInfoVO) {
        Long id = machineInfoService.createMachine(machineInfoVO);
        return success(id);
    }


    @PutMapping("/update")
    @Operation(summary = "编辑机器信息")
//    @PreAuthorize("@ss.hasPermission('ci:machineInfo:update')")
    public CommonResult<Boolean> updateMachineInfo(@Valid @RequestBody MachineInfoVO machineInfoVO) {
        machineInfoService.updateMachineInfo(machineInfoVO);
        return success(true);
    }

    @PutMapping("/updateStatus")
    @Operation(summary = "机器启用/停用")
//    @PreAuthorize("@ss.hasPermission('ci:machineInfo:status')")
    public CommonResult<Integer> updateStatus(@Valid @RequestBody MachineInfoVO machineInfoVO) {
        Integer newStatus = machineInfoService.updateStatus(machineInfoVO);
        return success(newStatus);
    }

    @PostMapping("/list")
    @Operation(summary = "获取机器信息列表")
    public CommonResult<PageResult<MachineInfoVO>> list(@Valid @RequestBody MachineInfoVO machineInfoVO) {
        PageResult<MachineInfoDO> pageResult = machineInfoService.listMachineInfo(machineInfoVO);
        if (CollUtil.isEmpty(pageResult.getList())) {
            return success(new PageResult<>(pageResult.getTotal()));
        }
        return success(BeanUtils.toBean(pageResult, MachineInfoVO.class));
    }

    @DeleteMapping("/delete")
    @Operation(summary = "机器信息删除")
//    @PreAuthorize("@ss.hasPermission('ci:machineInfo:delete')")
    public CommonResult<Boolean> deleteMachineInfo(@RequestParam("id") Long id) {
        machineInfoService.deleteMachineInfo(id);
        return success(true);
    }

    @DeleteMapping("/deleteList")
    @Operation(summary = "批量删除机器信息")
//    @PreAuthorize("@ss.hasPermission('ci:machineInfo:delete')")
    public CommonResult<Boolean> deleteMachineInfoList(@RequestParam("machineInfoIds") String ids) {
        machineInfoService.deleteMachineInfoList(ids);
        return success(true);
    }

    @PostMapping("/test")
    @Operation(summary = "测试机器连接")
    public CommonResult<Boolean> testConnection(@RequestParam("id") Long id) {
        return success(machineInfoService.testConnection(id));
    }

    @GetMapping("/status/{machineName}")
    @Operation(summary = "获取机器连接状态")
    public CommonResult<ConnectionStatus> getConnectionStatus(@PathVariable String machineName) {
        return success(machineInfoService.getConnectionStatus(machineName));
    }

    @GetMapping("/status/all")
    @Operation(summary = "获取所有机器连接状态")
    public CommonResult<Map<String, ConnectionStatus>> getAllConnectionStatus() {
        return success(machineInfoService.getAllConnectionStatus());
    }

    @PostMapping("/connect")
    @Operation(summary = "建立机器连接")
    public CommonResult<String> connect(@Valid @RequestBody MachineInfoDO machineInfoDO) {
        return success(machineInfoService.connect(machineInfoDO));
    }

    @GetMapping("/disconnect/{sessionId}")
    @Operation(summary = "断开机器连接")
    public CommonResult<Boolean> disconnect(@PathVariable String sessionId) {
        return success(machineInfoService.disconnect(sessionId));
    }

    @GetMapping("/execute/{sessionId}")
    @Operation(summary = "执行远程命令")
    public CommonResult<String> executeCommand(
        @PathVariable String sessionId,
        @RequestBody String command) {

        return success(machineInfoService.executeCommand(sessionId, command));
    }

    @GetMapping("/upload/{sessionId}")
    @Operation(summary = "上传文件到远程机器")
    public CommonResult<Boolean> uploadFile(
        @PathVariable String sessionId,
        @RequestParam String localFilePath,
        @RequestParam String remoteFilePath) {
        return success(machineInfoService.uploadFile(sessionId, localFilePath, remoteFilePath));
    }

    @GetMapping("/download/{sessionId}")
    @Operation(summary = "从远程机器下载文件")
    public CommonResult<Boolean> downloadFile(
        @PathVariable String sessionId,
        @RequestParam String remoteFilePath,
        @RequestParam String localFilePath) {
        return success(machineInfoService.downloadFile(sessionId, remoteFilePath, localFilePath));
    }
}
