package cd.casic.module.machine.service.impl;

import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.module.machine.controller.vo.SecretKeyVO;
import cd.casic.module.machine.enums.AuthenticationType;
import cd.casic.module.machine.enums.MachineInfoType;
import cd.casic.module.machine.handler.ConnectionSession;
import cd.casic.module.machine.dal.mysql.MachineInfoMapper;
import cd.casic.module.machine.controller.vo.MachineInfoVO;
import cd.casic.module.machine.dal.dataobject.MachineInfoDO;
import cd.casic.module.machine.enums.ConnectionStatus;
import cd.casic.module.machine.enums.MachineInfoStatus;
import cd.casic.module.machine.service.MachineInfoService;
import cd.casic.module.machine.service.SecretKeyService;
import com.google.common.annotations.VisibleForTesting;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import cd.casic.framework.commons.util.object.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

import static cd.casic.framework.commons.exception.util.ServiceExceptionUtil.exception;
import static cd.casic.module.machine.contants.MachineErrorCodeConstants.*;

/**
 * 机器信息服务实现类
 */
@Slf4j
@Service("machineInfoService")
public class MachineInfoServiceImpl implements MachineInfoService {

    @Resource
    private SecretKeyService secretKeyService;
    @Resource
    private MachineInfoMapper machineInfoMapper;
    @Resource
    private ConnectionSession connectionSession;
    /**
     * 会话ID生成器
     */
    private final AtomicInteger sessionIdGenerator = new AtomicInteger(1000);

    /**
     * 会话管理：会话ID -> 连接会话
     */
    private final Map<String, ConnectionSession> sessions = new ConcurrentHashMap<>();

    /**
     * 机器名称 -> 会话ID
     */
    private final Map<String, String> machineSessionMapping = new ConcurrentHashMap<>();

    @Override
    public Long createMachine(MachineInfoVO machineInfoVO) {
        validateMachineEnvAdd(machineInfoVO);
        validateMachineTagUnique(machineInfoVO.getTag());
        MachineInfoDO machineInfoDO = BeanUtils.toBean(machineInfoVO, MachineInfoDO.class);
        if (machineInfoVO.getAuthenticationType() == 2) {
            Long secretKeyId = machineInfoDO.getSecretKeyId();
            SecretKeyVO secretKey = secretKeyService.getSecretKey(secretKeyId);
            if (secretKey == null) {
                throw exception(SECRET_KEY_NOT_EXISTS);
            }
        }
        machineInfoMapper.insert(machineInfoDO);
        return machineInfoDO.getId();
    }

    @Override
    public void updateMachineInfo(MachineInfoVO machineInfoVO) {
        validateMachineEnvAdd(machineInfoVO);
        String newTag = machineInfoVO.getTag();
        MachineInfoDO machineInfoDO = validateMachineInfoExists(machineInfoVO.getId());
        String oldTag = machineInfoDO.getTag();
        if (!newTag.equals(oldTag)) {
            validateMachineTagUnique(newTag);
        }
        BeanUtils.copyProperties(machineInfoVO, machineInfoDO);
        machineInfoMapper.updateById(machineInfoDO);
    }

    @Override
    public Integer updateStatus(MachineInfoVO machineInfoVO) {
        machineInfoMapper.updateStatus(machineInfoVO.getId(), machineInfoVO.getStatus());
        return machineInfoVO.getStatus();
    }

    @Override
    public PageResult<MachineInfoDO> listMachineInfo(MachineInfoVO machineInfoVO) {
        return machineInfoMapper.selectPage(machineInfoVO);
    }

    @Override
    public void bindingSecretKey(List<Long> machineInfoIds, Long secretKeyId) {
        machineInfoMapper.bindingSecretKey(machineInfoIds, secretKeyId);
    }

    @Override
    @Transactional//其中一个在线，那么就回滚
    public void deleteMachineInfoList(String machineInfoIds) {
        List<Long> machineInfoIdList = Arrays.stream(machineInfoIds.split(","))
            .map(String::trim)
            .filter(s -> !s.isEmpty())
            .map(Long::parseLong)
            .toList();
        machineInfoIdList.forEach(this::deleteMachineInfo);
    }

    @Override
    public void deleteMachineInfo(Long machineInfoId) {
        MachineInfoDO machineInfoDO = validateMachineInfoExists(machineInfoId);
        validateMachineEnable(machineInfoDO);
        machineInfoMapper.deleteById(machineInfoId);
    }


    @Override
    public boolean testConnection(Long id) {
        //先查询机器是否存在，在判断机器可用性
        MachineInfoDO machineInfoDO = validateMachineInfoExists(id);
        validateMachineUnEnable(machineInfoDO);
        log.info("测试机器连接: {}", machineInfoDO.getHostIp());
        connectionSession.setMachineInfo(machineInfoDO);
        try {
            connectionSession.connect();
            return true;
        } catch (Exception e) {
            log.error("机器连接测试失败: {}", e.getMessage(), e);
            return false;
        }
    }

    @Override
    public ConnectionStatus getConnectionStatus(String machineName) {
        String sessionId = machineSessionMapping.get(machineName);
        if (sessionId == null) {
            return ConnectionStatus.DISCONNECTED;
        }

        ConnectionSession session = sessions.get(sessionId);
        return session != null ? session.getStatus() : ConnectionStatus.DISCONNECTED;
    }

    @Override
    public Map<String, ConnectionStatus> getAllConnectionStatus() {
        Map<String, ConnectionStatus> result = new HashMap<>();

        machineSessionMapping.forEach((machineName, sessionId) -> {
            ConnectionSession session = sessions.get(sessionId);
            result.put(machineName, session != null ? session.getStatus() : ConnectionStatus.DISCONNECTED);
        });

        return result;
    }

    @Override
    public String connect(MachineInfoDO machineInfoDO) {
        //先查询机器是否存在，在判断机器可用性
        validateMachineUnEnable(machineInfoDO);
        log.info("建立机器连接: {}", machineInfoDO.getHostIp());

        // 检查是否已连接
        String existingSessionId = machineSessionMapping.get(machineInfoDO.getName());
        if (existingSessionId != null) {
            ConnectionSession existingSession = sessions.get(existingSessionId);
            if (existingSession != null && existingSession.getStatus() == ConnectionStatus.CONNECTED) {
                log.info("机器已连接，返回现有会话: {}", machineInfoDO.getHostIp());
                return existingSessionId;
            }
        }

        try {
            connectionSession.setMachineInfo(machineInfoDO);

            connectionSession.connect();

            // 生成会话ID
            String sessionId = generateSessionId();

            // 保存会话
            sessions.put(sessionId, connectionSession);
            machineSessionMapping.put(machineInfoDO.getName(), sessionId);

            log.info("机器连接成功: {}, 会话ID: {}", machineInfoDO.getHostIp(), sessionId);
            return sessionId;
        } catch (Exception e) {
            log.error("机器连接失败: {}", e.getMessage(), e);
            throw new RuntimeException("机器连接失败: " + e.getMessage(), e);
        }
    }

    @Override
    public boolean disconnect(String sessionId) {
        log.info("断开机器连接: {}", sessionId);

        ConnectionSession session = sessions.get(sessionId);
        if (session == null) {
            log.warn("会话不存在: {}", sessionId);
            return false;
        }

        try {
            session.disconnect();

            // 清理会话
            sessions.remove(sessionId);
            machineSessionMapping.entrySet().removeIf(entry -> entry.getValue().equals(sessionId));

            log.info("机器连接已断开: {}", sessionId);
            return true;
        } catch (Exception e) {
            log.error("断开连接失败: {}", e.getMessage(), e);
            return false;
        }
    }

    @Override
    public String executeCommand(String sessionId, String command) {
        log.info("执行命令: {}, 会话ID: {}", command, sessionId);

        ConnectionSession session = sessions.get(sessionId);
        if (session == null) {
            throw new RuntimeException("会话不存在: " + sessionId);
        }

        if (session.getStatus() != ConnectionStatus.CONNECTED) {
            throw new RuntimeException("会话未连接: " + sessionId);
        }

        try {
            return session.executeCommand(command);
        } catch (Exception e) {
            log.error("命令执行失败: {}", e.getMessage(), e);
            throw new RuntimeException("命令执行失败: " + e.getMessage(), e);
        }
    }

    @Override
    public boolean uploadFile(String sessionId, String localFilePath, String remoteFilePath) {
        log.info("上传文件: {} -> {}, 会话ID: {}", localFilePath, remoteFilePath, sessionId);

        ConnectionSession session = sessions.get(sessionId);
        if (session == null) {
            throw new RuntimeException("会话不存在: " + sessionId);
        }

        if (session.getStatus() != ConnectionStatus.CONNECTED) {
            throw new RuntimeException("会话未连接: " + sessionId);
        }

        try {
            return session.uploadFile(localFilePath, remoteFilePath);
        } catch (Exception e) {
            log.error("文件上传失败: {}", e.getMessage(), e);
            throw new RuntimeException("文件上传失败: " + e.getMessage(), e);
        }
    }

    @Override
    public boolean downloadFile(String sessionId, String remoteFilePath, String localFilePath) {
        log.info("下载文件: {} -> {}, 会话ID: {}", remoteFilePath, localFilePath, sessionId);

        ConnectionSession session = sessions.get(sessionId);
        if (session == null) {
            throw new RuntimeException("会话不存在: " + sessionId);
        }

        if (session.getStatus() != ConnectionStatus.CONNECTED) {
            throw new RuntimeException("会话未连接: " + sessionId);
        }

        try {
            return session.downloadFile(remoteFilePath, localFilePath);
        } catch (Exception e) {
            log.error("文件下载失败: {}", e.getMessage(), e);
            throw new RuntimeException("文件下载失败: " + e.getMessage(), e);
        }
    }

    @VisibleForTesting
    void validateMachineEnvAdd(MachineInfoVO machineInfoVO) {
        if (machineInfoVO.getHostIp().isEmpty()) {
            throw exception(MACHINE_INFO_HOST_IP_NULL);
        }
        if (machineInfoVO.getUsername().isEmpty()) {
            throw exception(MACHINE_INFO_USER_NAME_NULL);
        }
        if (machineInfoVO.getTag().isEmpty()) {
            throw exception(MACHINE_INFO_TAG_NULL);
        }

        if (machineInfoVO.getAuthenticationType() != null) {
            boolean flag = true;
            for (int type : AuthenticationType.ARRAYS) {
                if (type == machineInfoVO.getAuthenticationType()) {
                    flag = false;
                    break;
                }
            }
            if (flag) {
                throw exception(MACHINE_INFO_AUTHENTICATION_TYPE_NOT_EXISTS);
            }
        } else {
            throw exception(MACHINE_INFO_AUTHENTICATION_TYPE_NULL);
        }


        if (machineInfoVO.getMachineInfoType() != null) {
            boolean flag = true;
            for (int type : MachineInfoType.ARRAYS) {
                if (type == machineInfoVO.getMachineInfoType()) {
                    flag = false;
                    break;
                }
            }
            if (flag) {
                throw exception(MACHINE_INFO_TYPE_NOT_EXISTS);
            }
        } else {
            throw exception(MACHINE_INFO_TYPE_NULL);
        }

    }

    @VisibleForTesting
    void validateMachineTagUnique(String tag) {
        if (machineInfoMapper.existsByTag(tag)) {
            throw exception(MACHINE_INFO_TAG_EXISTS);
        }
    }

    @VisibleForTesting
    MachineInfoDO validateMachineInfoExists(Long id) {
        if (id == null) {
            throw exception(MACHINE_INFO_NULL);
        }
        MachineInfoDO machineInfoDO = machineInfoMapper.selectById(id);
        if (machineInfoDO == null) {
            throw exception(MACHINE_INFO_NULL);
        }
        return machineInfoDO;
    }

    @VisibleForTesting
    void validateMachineEnable(MachineInfoDO machineInfoDO) {
        if (machineInfoDO.getStatus() == MachineInfoStatus.ENABLE.getCode()) {
            throw exception(MACHINE_ENABLE);
        }
    }

    @VisibleForTesting
    void validateMachineUnEnable(MachineInfoDO machineInfoDO) {

        if (machineInfoDO.getStatus() == MachineInfoStatus.UN_ENABLE.getCode()) {
            throw exception(MACHINE_UN_ENABLE);
        }
    }

    /**
     * 生成会话ID
     */
    private String generateSessionId() {
        return "session-" + sessionIdGenerator.incrementAndGet();
    }
}
